<?php

namespace Service;

use Exception\CustomException;
use Util\Messages;
use Util\Utilities;
use Doctrine\ORM\EntityManager;
use \Zeedhi\DevOps\Util\Utilities as DOUtilities;
use Zeedhi\DevOps\Model\Devops\Entities\Mnemonic;
use Zeedhi\DevOps\Model\Devops\Entities\ServerMnemonic;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\Product;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\ProductFiles;
use Zeedhi\DevOps\Model\Servermanager\Entities\Server as ServerEntity;
use Util\InstanceProvider;

class Server {
    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var Utilities $utilities */
    protected $utilities;
    /** @var \Factory\Server $serverFactory */
    protected $serverFactory;
    /** @var \Factory\Mnemonic $mnemonicFactory */
    protected $mnemonicFactory;
    /** @var \Zeedhi\DevOps\Model\Servermanager\Repositories\Server $serverRepository */
    protected $serverRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Server $resourceRepository */
    protected $resourceRepository;
    /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\Product $productRepository */
    protected $productRepository;
    /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\ProductFiles $productRepository */
    protected $productFilesRepository;
    /** @var \Zeedhi\DevOps\Model\DevOps\Repositories\Mnemonic $mnemonicRepository */
    protected $mnemonicRepository;
    /** @var \Zeedhi\DevOps\Model\DevOps\Repositories\ServerMnemonic $serverMnemonicRepository */
    protected $serverMnemonicRepository;
    /** @var \Zeedhi\DevOps\Model\Servermanager\Repositories\ServerAccessProfile $serverAccessProfileRepository */
    protected $serverAccessProfileRepository;

    /**
     * @param EntityManager $entityManager
     * @param Utilities $utilities
     * @param \Factory\Server $serverFactory
     * @param \Factory\Mnemonic $mnemonicFactory
     */
    public function __construct(EntityManager $entityManager, Utilities $utilities,
                                \Factory\Server $serverFactory, \Factory\Mnemonic $mnemonicFactory, \Factory\MnemonicGrouping $mnemonicGroupingFactory,
                                \Factory\ServerAccessProfile $serverAccessProfileFactory) {
        $this->utilities     = $utilities;
        $this->entityManager = $entityManager;
        $this->serverFactory = $serverFactory;
        $this->mnemonicFactory = $mnemonicFactory;
        $this->mnemonicGroupingFactory = $mnemonicGroupingFactory;
        $this->serverAccessProfileFactory = $serverAccessProfileFactory;

        $this->mnemonicRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Devops\Entities\Mnemonic');
        $this->mnemonicGroupingRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Devops\Entities\MnemonicGrouping');
        $this->serverRepository   = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Servermanager\Entities\Server');
        $this->resourceRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        $this->productRepository  = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Configurationmanager\Entities\Product');
        $this->serverMnemonicRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Devops\Entities\ServerMnemonic');
        $this->productFilesRepository   = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Configurationmanager\Entities\ProductFiles');
        $this->organizationRepository = $this->entityManager->getRepository('Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization');
        $this->serverAccessProfileRepository   = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Servermanager\Entities\ServerAccessProfile');
    }

    /**
     * @param $conditions
     * @return array
     */
    public function findAvailableCustomerServers($conditions) {
        $customerCode = $this->utilities->getCurrentCustomer()->getCdcliente();
        $filterQuery = DOUtilities::createQueryFiltered($conditions);
        return $this->serverRepository->findAvailableCustomerServers($customerCode, $filterQuery);
    }

    public function findAllowedProfiles($serverId) {
        $profiles = $this->serverAccessProfileRepository->findAllowedProfiles($this->utilities->getCurrentOrganizationId(), $serverId);
        return DOUtilities::getArrayFromArrayAssociative($profiles,"NRPERFILACESSO");
    }

    /**
     * @param $id
     * @param $name
     * @param $wsEmail
     * @param $backupLimit
     * @param $description
     * @param $isActive
     * @param $isWindows
     * @return int
     * @throws \Exception
     */
    public function saveAvailableCustomerServer($id, $name, $wsEmail, $backupLimit, $description, $isActive, $isWindows, $isRequiredTicket, $allowedProfiles) {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();

            $server = !empty($id) ? $this->serverRepository->find($id) : null;
            $customerCode = $this->utilities->getCurrentCustomer()->getCdcliente();
            $resource = $this->resourceRepository->findOneBy(array('externalCode' => $customerCode));

            $server = $this->serverFactory->save($server, $name, $wsEmail, $resource, $backupLimit, $description, $isWindows, $isActive,$isRequiredTicket);
            $this->entityManager->persist($server);

            $orgId = $this->utilities->getCurrentOrganizationId();
            $organization = $this->organizationRepository->find($orgId);
            $serverAccess = $this->serverAccessProfileRepository->findBy(array("organization" => $organization, "server" => $server));
            foreach ($serverAccess as $serverRemove){
                $this->entityManager->remove($serverRemove);
            }
            foreach($allowedProfiles as $nrperfilacesso){
                $serverAccess = $this->serverAccessProfileFactory->save($server, $organization, $nrperfilacesso);
                $this->entityManager->persist($serverAccess);
            }

            $this->entityManager->flush();
            $connection->commit();

            return $server->getId();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $id
     * @throws \Exception
     */
    public function removeAvailableCustomerServer($id) {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();

            $server = $this->serverRepository->find($id);
            $this->entityManager->remove($server);

            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $operation
     * @param $serverId
     * @param $productId
     * @return array
     */
    public function validateServerPermissions($operation, $serverId, $productId) {
        /** @var Product $product */
        $product = $this->productRepository->find($productId);
        /** @var ServerEntity $server */
        $server  = $this->serverRepository->find($serverId);
        $types   = self::getScriptTypesByOperation($operation);

        return $this->getNeedApproval($product, $server, $types);
    }

    /**
     * @param Product $product
     * @param ServerEntity $server
     * @param $scripTypes
     * @param array $needsApproval
     * @return array
     */
    private function getNeedApproval(Product $product, ServerEntity $server = null, $scripTypes, $needsApproval = array()) {
        if ($server && $server->getOnlyApprovedScripts() == 'Y') {
            /** @var ProductFiles[] $productFiles */
            $productFiles = $this->productFilesRepository->findBy(array('product' => $product, 'type' => $scripTypes));

            foreach ($productFiles as $productFile) {
                if ($productFile->getState() == ProductFiles::SCRIPT_PENDING) {
                    $script = array('name' => $productFile->getFileName(), 'type' => $productFile->getType());
                    if (!in_array($script, $needsApproval)) {
                        $needsApproval[] = $script;
                    }
                }
            }
            return $needsApproval;
        }
        return $needsApproval;
    }

    /**
     * @return array
     */
    public static function scriptTypes(){
        return array(
            ServerEntity::APPROVED_DEPLOY => array(
                ProductFiles::DEPLOY_SCRIPT_AFTER,
                ProductFiles::DEPLOY_SCRIPT_BEFORE,
                ProductFiles::BACKUP_SCRIPT_AFTER,
                ProductFiles::BACKUP_SCRIPT_BEFORE,
            ),
            ServerEntity::APPROVED_BACKUP => array(
                ProductFiles::BACKUP_SCRIPT_AFTER,
                ProductFiles::BACKUP_SCRIPT_BEFORE,
            )
        );
    }

    /**
     * @param $index
     * @return mixed
     */
    public static function getScriptTypesByOperation($index){
        return self::scriptTypes()[$index];
    }

    /**
     * @param $operation
     * @param $serverId
     * @param $productId
     * @return array
     */
    public function validateServerMnemonic($operation, $serverId, $productId, &$mnemonicsarray = array(), &$productsIds = array()) {
        /** @var Product $product */
        $product = $this->productRepository->find($productId);
        /** @var ServerEntity $server */
        $server  = $this->serverRepository->find($serverId);
        $types   = self::getScriptTypesByOperation($operation);
        $productFiles   = $this->productFilesRepository->findBy(array('product' => $product, 'type' => $types));
        $productFileIds = Utilities::getArrayByArrayObjectWithGet($productFiles);
        return $this->validateMnemonic($productFileIds, $server, null, $mnemonicsarray, $product, $productsIds);
    }

    public function validateServerProductRelationship($productsIds = array(), &$serverMnemonic, $serverId) {
        $unRelatedServer = array();
        foreach($productsIds as $key => $productId) {
            $relatedProduct = $this->mnemonicRepository->validateProductServerLinked($serverId, $productId);
            if(empty($relatedProduct)) {
                array_push($unRelatedServer, $serverMnemonic[$key]);
            }
        }

        if(!empty($unRelatedServer)) {
            $arrayAux[0]    = "ERRO_MNEMONIC_SERVER_PRODUCT_RELATIONSHIP";
            $arrayAux[1]    = $unRelatedServer;
            $serverMnemonic = $arrayAux;
        } else {
            $arrayAux[0]    = "ERRO_MNEMONIC_NOT_VALUE";
            $arrayAux[1]    = $serverMnemonic;
            $serverMnemonic = $arrayAux;
        }
    }

    /**
     * @param $productFileIds
     * @param ServerEntity $server
     * @param array $mnemonicsPerServer
     * @return array
     */
    protected function validateMnemonic($productFileIds, ServerEntity $server, $mnemonicsPerServer = array(), &$mnemonicsarray = array(), $product, &$productsIds) {
        $mnemonics = $this->mnemonicRepository->findMnemonicNotValueByProductFilesAndServer($productFileIds, $server->getId());
        if (!empty($mnemonics)) {
            array_push($mnemonicsarray,$product->getName());
            array_push($productsIds,$product->getId());
        }
        return $mnemonicsPerServer;
    }

    /**
     * @param $serverId
     * @param $conditions
     * @param $productIds
     * @return array
     */
    public function findMnemonicByServer($serverId, $productIds, $conditions) {
        $productFilter = empty($productIds) ? '' : "AND ". DOUtilities::fixBugWhereINQuery($productIds, "MN.PRODUCT_ID");
        $filterQuery   = DOUtilities::createQueryFiltered($conditions);
        return $this->mnemonicRepository->findMnemonicByServer($serverId, $filterQuery, $productFilter);
    }

    public function findMnemonicGrouping($conditions, $page, $pageSize) {
        $filters = DOUtilities::createQueryFiltered($conditions);
        $limit   = $page * $pageSize;
        $offset  = ($page - 1) * $pageSize;
        $organizationId = $this->utilities->getCurrentOrganizationId();
        return $this->mnemonicGroupingRepository->findMnemonicGrouping($organizationId, $filters, $limit, $offset);
    }


    /**
     * @param $serverId
     * @param $conditions
     * @return array
     */
    public function findProductsByServer($serverId, $conditions) {
        $filterQuery = DOUtilities::createQueryFiltered($conditions);
        return $this->productRepository->getProductsByServer($serverId, $filterQuery);
    }

    /**
     * @param $serverId
     * @param $mnemonicId
     * @param $mnemonicString
     * @param $value
     * @param $label
     * @return int
     * @throws \Exception
     */
    public function saveMnemonicByServer($serverId, $mnemonicId, $mnemonicString, $value, $label, $isEncrypted, $mnemonicGroup = null) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var ServerEntity $server */
            $server = $this->serverRepository->find($serverId);
            /** @var Mnemonic $mnemonic */
            $mnemonic = $this->mnemonicRepository->findOneBy(array('id' => $mnemonicId));
            
            $serverMnemonic = $this->serverMnemonicRepository->findOneBy(array('mnemonic' => $mnemonic, 'server' => $server));
            $oldValue = $serverMnemonic->getValue();
            $idMnemonic = $mnemonic->getId();

            if (!empty($value)){
                $idMnemonic = $this->saveMnemonic($server, $mnemonic, $mnemonicString, $value, $label, $isEncrypted, $mnemonicGroup);
            } else if (empty($value) && empty($mnemonic->getDefaultValue())){
                throw new CustomException('ERR_MNEMONIC_VALUE_NOT_INFORMED');
            } else if (empty($value) && $serverMnemonic) {
                $this->entityManager->remove($serverMnemonic);
                $this->entityManager->flush();
            }

            $this->entityManager->flush();
            $date = new \DateTime();
            $mongoRow = array(
                'MNEMONIC_SERVER_ID'  => intval($serverMnemonic->getId()),
                'OLD_VALUE'     => $oldValue,
                'NEW_VALUE' => $value,
                'NMOPERADOR'  => Utilities::getEnvironment()->getCurrentUser()->getNmoperador(),
                "DATE" => intval($date->format('YmdHis'))
            );
            InstanceProvider::getDevopsMongoDB()->insert("MNEMONIC_SERVER_LOG", $mongoRow);            
            
            $connection->commit();

            return $idMnemonic;
        } catch (\Exception $ex) {
            $connection->rollBack();
            throw $ex;
        }
    }

    /**
     * @param ServerEntity $server
     * @param Mnemonic $mnemonic
     * @param $mnemonicString
     * @param null $value
     * @param null $label
     * @return int
     */
    public function saveMnemonic(ServerEntity $server, Mnemonic $mnemonic = null, $mnemonicString,
                                 $value = null, $label = null, $isEncrypted = 'N', $mnemonicGroup = null){
        if(empty($mnemonic)){
            $mnemonic = $this->mnemonicFactory->saveMnemonic(null, $mnemonicString, $label, $mnemonicGroup);
            $this->entityManager->persist($mnemonic);
            $this->entityManager->flush();
        } else {
            $mnemonic->setLabel($label);
            $mnemonic->setMnemonicGroupingId($mnemonicGroup);
            $this->entityManager->persist($mnemonic);
        }
        /** @var ServerMnemonic $serverMnemonic */
        $serverMnemonic = $this->serverMnemonicRepository->findOneBy(array('mnemonic' => $mnemonic, 'server' => $server));
        if (empty($serverMnemonic)) {
            $serverMnemonic = $this->mnemonicFactory->createServerMnemonic($mnemonic, $server, $value);
            $this->entityManager->persist($serverMnemonic);
            $this->entityManager->flush();
        }
        if($isEncrypted == 'Y' && $value == '**********'){
            $serverMnemonic->setValue($serverMnemonic->getValue());
        }else if($isEncrypted == 'Y' && strtoupper($value) != 'NULL'){
            $serverMnemonic->setValue(Utilities::encrypt($value));
        } else {
            $serverMnemonic->setValue($value);
        }
        $this->entityManager->persist($serverMnemonic);
        $this->entityManager->flush();
        return $mnemonic->getId();
    }
}
